﻿#!/usr/bin/env python3
# -*- coding:utf-8 -*-
#-------------------------------------------------------------------------------
# Name:        module1
# Purpose:      時刻と時間幅を記載したファイルに基づき、RTKLIBが出力したpos（測位結果）を統計処理を実施する。
#               静止中の測位結果をまとめるのが目的です。
#               記載された時刻の間だけの測位結果をまとめます。
#               時刻の指定は複数可能です。
#               また、時刻が重なっている場合でも処理可能です。
#
# Author:      morishita
#
# Created:     11/01/2013
# Copyright:   (c) morishita 2013
# Licence:     new BSD
#-------------------------------------------------------------------------------

import os
import sys
import re
import RtklibPosProcessing
import mathKM
import timeKM
import datetime

def main():
    """ 指定された*.posファイル（RTKLIBが出力した測位結果ファイル）を、指定された時刻のリストに基づき統計処理を実施します。
    第1引数：RTKLIBの測位演算ファイル（*.pos）のパス
    第2引数：時間のリストファイルのパス
    """
    argvs = sys.argv  # コマンドライン引数を格納したリストの取得
    argc = len(argvs) # 引数の個数
    # デバッグプリント
    print(argvs)
    print(argc)
    #print
    if (argc != 3):                                     # 引数が足りない場合は、その旨を表示
        print('Usage: # python %s filename' % argvs[0]) # 0はスクリプトファイル自体を指す
        quit()                                          # プログラムの終了
    fnameOfRtklib    = argvs[1]
    if os.path.exists(fnameOfRtklib) == False:          # ファイルの存在が確認できない場合は終了
        quit()
    if os.path.isfile(fnameOfRtklib) != True:           # ファイルでなければ終了
        quit()
    fnameOfTimeList = argvs[2]
    if os.path.exists(fnameOfTimeList) == False:        # ファイルの存在が確認できない場合は終了
        quit()
    if os.path.isfile(fnameOfTimeList) != True:         # ファイルでなければ終了
        quit()
    # タイムリスト読み込み
    fr = open(fnameOfTimeList,"r")
    times = fr.readlines()
    fr.close()
    if len(times) == 0:
        quit()
    # RTKLIBの測位データ読み込み
    fr = open(fnameOfRtklib,"r")
    poss = fr.readlines()
    fr.close()
    if len(poss) == 0:
        quit()
    #root, ext = os.path.splitext(fnameOfRtklib)          # ファイル名と拡張子に分ける。拡張子には.が入っている

    # 日付および時間幅(正確には数値なら何でも合致するのだが)に一致する正規表現モジュール
    # [yyyy/MM/dd hh:mm:ss]または[yyyy-MM-dd hh:mm:ss]にヒットする。(/|-)で、/か-で区切られた日付にヒットする。
    timeListPattern = re.compile(r'(?P<date>(?P<year>\d{4})(?:/|-)(?P<month>\d{1,2})(?:/|-)(?P<day>\d{1,2}))\ (?P<clock>(?P<hour>\d{1,2}):(?P<minute>\d{1,2}):(?P<second>\d{1,2}[.]?\d*))(\t|,)(?P<timeWidth>\d+)')
    index = 0
    fw = open("log.csv", "w")
    fw2 = open("log lite.csv", "w")
    fw.write("file name,開始時刻,時間幅[s],経度の統計情報,,,,緯度の統計情報,,,,高さの統計情報,,,,統計に使用されたデータ件数,抽出率,抽出された中での各Q値の割合,,,,," + "\n")
    fw.write("file name,start time,time width,mean,sd,use rate,distribution bias, mean,sd,use rate,distribution bias,mean,sd,use rate,distribution bias,統計に使用されたデータ件数,抽出率,Q=0,Q=1,Q=2,Q=3,Q=4,Q=5" + "\n")
    for line in times:                                          # 時間のリストを1行ずつ処理する
        print("index: {0}/{1}".format(index + 1, len(times)))
        matchTest = timeListPattern.search(line)
        if matchTest != None:
            time = timeKM.getTime(line)
            tw   = int(matchTest.group('timeWidth'))
            t1 = time
            t2 = time + datetime.timedelta(seconds=tw)
            print("now: {0} to {1}".format(t1, t2))
            posList = []                                        # 全データを毎回走査して、指定された時刻のデータのみ集める
            for pos in poss:
                matchTest = RtklibPosProcessing.rePosPattern.search(pos)
                if matchTest != None:
                    t = timeKM.getTime(matchTest.group('time'))
                    if t1 <= t <= t2:
                        posList.append(pos)
                    if t > t2:                                  # データが時系列なら、抜けたほうが処理が早くなる
                        break
            if len(posList) > 0:
                result = RtklibPosProcessing.getStatistics(posList, 1, 2)
                print(result)
                if result == None:
                    fw2.write(os.path.basename(fnameOfRtklib) + ",\n")   # 後処理のために、空欄とする
                    fw.write(os.path.basename(fnameOfRtklib) + ",\n")
                else:
                    out = str(result)
                    out = re.sub("[\'()\[\]]", "", out)
                    fw.write(os.path.basename(fnameOfRtklib) + "," + out + "\n")
                    log = ""
                    log += "," + str(result[0])
                    log += "," + str(result[1])
                    log += "," + str(result[2][0][0])           # lon
                    log += "," + str(result[3][0][0])           # lat
                    log += "," + str(result[4][0][0])           # height
                    log += "," + str(result[2][0][1])           # lon sd
                    log += "," + str(result[3][0][1])           # lat sd
                    log += "," + str(result[4][0][1])           # height sd
                    fw2.write(os.path.basename(fnameOfRtklib) + log + "\n")
        index += 1
    fw.close()
    fw2.close()
    return

if __name__ == '__main__':
    main()
